@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006 Linpro AS
+ * Copyright (c) 2006-2007 Linpro AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -14,7 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  *
- * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
+ * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
@@ -26,16 +26,17 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: cache.h 1177 2006-10-18 14:27:06Z des $
+ * $Id: cache.h 2147 2007-10-19 11:42:26Z des $
  */
 
 #include <sys/time.h>
 #include <sys/uio.h>
+#include <sys/socket.h>
 
 #include <pthread.h>
 #include <stdint.h>
 
-#include "queue.h"
+#include "vqueue.h"
 #include "vsb.h"
 
 #include "libvarnish.h"
@@ -55,6 +56,7 @@ enum {
 	HTTP_HDR_MAX = 32
 };
 
+/* Note: intentionally not IOV_MAX */
 #define MAX_IOVS	(HTTP_HDR_MAX * 2)
 
 /* Amount of per-worker logspace */
@@ -75,9 +77,27 @@ enum step {
 #undef STEP
 };
 
+/*--------------------------------------------------------------------
+ * Workspace structure for quick memory allocation.
+ */
+
+struct ws {
+	char			*s;		/* (S)tart of buffer */
+	char			*e;		/* (E)nd of buffer */
+	char			*f;		/* (F)ree pointer */
+	char			*r;		/* (R)eserved length */
+};
+
+void WS_Init(struct ws *ws, void *space, unsigned len);
+unsigned WS_Reserve(struct ws *ws, unsigned bytes);
+void WS_Release(struct ws *ws, unsigned bytes);
+void WS_ReleaseP(struct ws *ws, char *ptr);
+void WS_Assert(struct ws *ws);
+void WS_Reset(struct ws *ws);
+char *WS_Alloc(struct ws *ws, unsigned bytes);
+
 /*--------------------------------------------------------------------
  * HTTP Request/Response/Header handling structure.
- * RSN: struct worker and struct session will have one of these embedded.
  */
 
 struct http_hdr {
@@ -85,22 +105,22 @@ struct http_hdr {
 	char			*e;
 };
 
+enum httpwhence {
+	HTTP_Rx,
+	HTTP_Tx,
+	HTTP_Obj
+};
+
 struct http {
 	unsigned		magic;
 #define HTTP_MAGIC		0x6428b5c9
 
-	char			*s;		/* (S)tart of buffer */
-	char			*t;		/* start of (T)railing data */
-	char			*v;		/* end of (V)alid bytes */
-	char			*f;		/* first (F)ree byte */
-	char			*e;		/* (E)nd of buffer */
+	struct ws		ws[1];
+	char			*rx_s, *rx_e;	/* Received Request */
+	char			*pl_s, *pl_e;	/* Pipelined bytes */
 
 	unsigned char		conds;		/* If-* headers present */
-	enum httpwhence {
-		HTTP_Rx,
-		HTTP_Tx,
-		HTTP_Obj
-	}			logtag;
+	enum httpwhence 	logtag;
 
 	struct http_hdr		hd[HTTP_HDR_MAX];
 	unsigned char		hdf[HTTP_HDR_MAX];
@@ -111,7 +131,7 @@ struct http {
 /*--------------------------------------------------------------------*/
 
 struct acct {
-	time_t			first;
+	double			first;
 	uint64_t		sess;
 	uint64_t		req;
 	uint64_t		pipe;
@@ -129,11 +149,11 @@ struct worker {
 	struct objhead		*nobjhead;
 	struct object		*nobj;
 
-	time_t			idle;
+	double			used;
 
 	int			pipe[2];
 
-	TAILQ_ENTRY(worker)	list;
+	VTAILQ_ENTRY(worker)	list;
 	struct workreq		*wrq;
 
 	int			*wfd;
@@ -145,13 +165,14 @@ struct worker {
 	struct VCL_conf		*vcl;
 	struct srcaddr		*srcaddr;
 	struct acct		acct;
-	unsigned char		wlog[WLOGSPACE];
+
 	unsigned char		*wlp, *wle;
 	unsigned		wlr;
+	unsigned char		wlog[WLOGSPACE];
 };
 
 struct workreq {
-	TAILQ_ENTRY(workreq)	list;
+	VTAILQ_ENTRY(workreq)	list;
 	struct sess		*sess;
 };
 
@@ -159,15 +180,21 @@ struct workreq {
 
 /* Backend Connection ------------------------------------------------*/
 
+struct bereq {
+	unsigned		magic;
+#define BEREQ_MAGIC		0x3b6d250c
+	VTAILQ_ENTRY(bereq)	list;
+	void			*space;
+	unsigned		len;
+	struct http		http[1];
+};
+
 struct vbe_conn {
 	unsigned		magic;
 #define VBE_CONN_MAGIC		0x0c5e6592
-	TAILQ_ENTRY(vbe_conn)	list;
+	VTAILQ_ENTRY(vbe_conn)	list;
 	struct backend		*backend;
 	int			fd;
-	struct http		*http;
-	struct http		*http2;
-	struct http		http_mem[2];
 };
 
 /* Storage -----------------------------------------------------------*/
@@ -175,7 +202,7 @@ struct vbe_conn {
 struct storage {
 	unsigned		magic;
 #define STORAGE_MAGIC		0x1a4e51c0
-	TAILQ_ENTRY(storage)	list;
+	VTAILQ_ENTRY(storage)	list;
 	struct stevedore	*stevedore;
 	void			*priv;
 
@@ -205,6 +232,8 @@ struct object {
 	unsigned		xid;
 	struct objhead		*objhead;
 
+	unsigned char		*vary;
+
 	unsigned		heap_idx;
 	unsigned		ban_seq;
 
@@ -218,20 +247,23 @@ struct object {
 	unsigned		busy;
 	unsigned		len;
 
-	time_t			age;
-	time_t			entered;
-	time_t			ttl;
+	double			age;
+	double			entered;
+	double			ttl;
 
-	time_t			last_modified;
+	double			last_modified;
 
 	struct http		http;
-	TAILQ_ENTRY(object)	list;
+	VTAILQ_ENTRY(object)	list;
 
-	TAILQ_ENTRY(object)	deathrow;
+	VTAILQ_ENTRY(object)	deathrow;
 
-	TAILQ_HEAD(, storage)	store;
+	VTAILQ_HEAD(, storage)	store;
 
-	TAILQ_HEAD(, sess)	waitinglist;
+	VTAILQ_HEAD(, sess)	waitinglist;
+
+	double			lru_stamp;
+	VTAILQ_ENTRY(object)	lru;
 };
 
 struct objhead {
@@ -240,7 +272,7 @@ struct objhead {
 	void			*hashpriv;
 
 	pthread_mutex_t		mtx;
-	TAILQ_HEAD(,object)	objects;
+	VTAILQ_HEAD(,object)	objects;
 };
 
 /* -------------------------------------------------------------------*/
@@ -254,8 +286,10 @@ struct sess {
 
 	struct worker		*wrk;
 
-	unsigned		sockaddrlen;
+	socklen_t		sockaddrlen;
+	socklen_t		mysockaddrlen;
 	struct sockaddr		*sockaddr;
+	struct sockaddr		*mysockaddr;
 
 	/* formatted ascii client address */
 	char			addr[TCP_ADDRBUFSIZE];
@@ -266,21 +300,22 @@ struct sess {
 	const char		*doclose;
 	struct http		*http;
 
-	struct timespec		t_open;
-	struct timespec		t_req;
-	struct timespec		t_resp;
-	struct timespec		t_end;
+	/* Timestamps, all on TIM_real() timescale */
+	double			t_open;
+	double			t_req;
+	double			t_resp;
+	double			t_end;
 
 	enum step		step;
 	unsigned 		handling;
 	unsigned char		wantbody;
 	int			err_code;
-	const char		*err_expl;
+	const char		*err_reason;
 
-	TAILQ_ENTRY(sess)	list;
+	VTAILQ_ENTRY(sess)	list;
 
-	struct vbe_conn		*vbc;
 	struct backend		*backend;
+	struct bereq		*bereq;
 	struct object		*obj;
 	struct VCL_conf		*vcl;
 
@@ -289,6 +324,12 @@ struct sess {
 
 	struct workreq		workreq;
 	struct acct		acct;
+
+	/* pointers to hash string components */
+	unsigned		nhashptr;
+	unsigned		ihashptr;
+	unsigned		lhashptr;
+	const char		**hashptr;
 };
 
 struct backend {
@@ -301,7 +342,10 @@ struct backend {
 	struct addrinfo		*addr;
 	struct addrinfo		*last_addr;
 
-	TAILQ_HEAD(,vbe_conn)	connlist;
+	VTAILQ_HEAD(,vbe_conn)	connlist;
+
+	double			dnsttl;
+	double			dnstime;
 #if 0
 	double			responsetime;
 	double			timeout;
@@ -319,14 +363,18 @@ void vca_return_session(struct sess *sp);
 void vca_close_session(struct sess *sp, const char *why);
 void VCA_Prep(struct sess *sp);
 void VCA_Init(void);
+extern int vca_pipes[2];
 
 /* cache_backend.c */
 void VBE_Init(void);
 struct vbe_conn *VBE_GetFd(struct sess *sp);
-void VBE_ClosedFd(struct worker *w, struct vbe_conn *vc, int already);
+void VBE_ClosedFd(struct worker *w, struct vbe_conn *vc);
 void VBE_RecycleFd(struct worker *w, struct vbe_conn *vc);
+struct bereq *vbe_new_bereq(void);
+void vbe_free_bereq(struct bereq *bereq);
 
 /* cache_ban.c */
+void AddBan(const char *);
 void BAN_Init(void);
 void cli_func_url_purge(struct cli *cli, char **av, void *priv);
 void BAN_NewObj(struct object *o);
@@ -343,12 +391,15 @@ void CLI_Init(void);
 void EXP_Insert(struct object *o);
 void EXP_Init(void);
 void EXP_TTLchange(struct object *o);
+void EXP_Terminate(struct object *o);
 
 /* cache_fetch.c */
-int FetchBody(struct sess *sp);
-int FetchHeaders(struct sess *sp);
+int Fetch(struct sess *sp);
 
 /* cache_hash.c */
+void HSH_Prealloc(struct sess *sp);
+int HSH_Compare(struct sess *sp, const char *b, const char *e);
+void HSH_Copy(struct sess *sp, char *b, const char *e);
 struct object *HSH_Lookup(struct sess *sp);
 void HSH_Unbusy(struct object *o);
 void HSH_Ref(struct object *o);
@@ -356,22 +407,26 @@ void HSH_Deref(struct object *o);
 void HSH_Init(void);
 
 /* cache_http.c */
+const char *http_StatusMessage(int);
 void HTTP_Init(void);
 void http_ClrHeader(struct http *to);
-void http_CopyHttp(struct http *to, struct http *fm);
 unsigned http_Write(struct worker *w, struct http *hp, int resp);
-void http_GetReq(struct worker *w, int fd, struct http *to, struct http *fm);
-void http_CopyReq(struct worker *w, int fd, struct http *to, struct http *fm);
-void http_CopyResp(struct worker *w, int fd, struct http *to, struct http *fm);
-void http_SetResp(struct worker *w, int fd, struct http *to, const char *proto, const char *status, const char *response);
-void http_FilterHeader(struct worker *w, int fd, struct http *to, struct http *fm, unsigned how);
+void http_CopyResp(struct http *to, struct http *fm);
+void http_SetResp(struct http *to, const char *proto, const char *status, const char *response);
+void http_FilterFields(struct worker *w, int fd, struct http *to, struct http *fm, unsigned how);
+void http_FilterHeader(struct sess *sp, unsigned how);
+void http_PutProtocol(struct worker *w, int fd, struct http *to, const char *protocol);
+void http_PutStatus(struct worker *w, int fd, struct http *to, int status);
+void http_PutResponse(struct worker *w, int fd, struct http *to, const char *response);
 void http_PrintfHeader(struct worker *w, int fd, struct http *to, const char *fmt, ...);
 void http_SetHeader(struct worker *w, int fd, struct http *to, const char *hdr);
+void http_SetH(struct http *to, unsigned n, const char *fm);
 void http_Setup(struct http *ht, void *space, unsigned len);
 int http_GetHdr(struct http *hp, const char *hdr, char **ptr);
 int http_GetHdrField(struct http *hp, const char *hdr, const char *field, char **ptr);
 int http_GetStatus(struct http *hp);
-int http_IsBodyless(struct http *hp);
+const char *http_GetReq(struct http *hp);
+const char *http_GetProto(struct http *hp);
 int http_HdrIs(struct http *hp, const char *hdr, const char *val);
 int http_GetTail(struct http *hp, unsigned len, char **b, char **e);
 int http_Read(struct http *hp, int fd, void *b, unsigned len);
@@ -382,15 +437,14 @@ int http_RecvHead(struct http *hp, int fd);
 int http_DissectRequest(struct worker *w, struct http *sp, int fd);
 int http_DissectResponse(struct worker *w, struct http *sp, int fd);
 void http_DoConnection(struct sess *sp);
+void http_CopyHome(struct worker *w, int fd, struct http *hp);
+void http_Unset(struct http *hp, const char *hdr);
+
 
 #define HTTPH(a, b, c, d, e, f, g) extern char b[];
 #include "http_headers.h"
 #undef HTTPH
 
-/* cache_pass.c */
-int PassSession(struct sess *sp);
-void PassBody(struct sess *sp);
-
 /* cache_pipe.c */
 void PipeSession(struct sess *sp);
 
@@ -429,15 +483,31 @@ void WSL_Flush(struct worker *w);
 #endif
 
 /* cache_response.c */
-void RES_Error(struct sess *sp, int code, const char *expl);
+void RES_BuildHttp(struct sess *sp);
+void RES_Error(struct sess *sp, int code, const char *reason);
 void RES_WriteObj(struct sess *sp);
 
+/* cache_synthetic.c */
+void SYN_ErrorPage(struct sess *sp, int status, const char *reason, int ttl);
+
+/* cache_vary.c */
+void VRY_Create(struct sess *sp);
+int VRY_Match(struct sess *sp, unsigned char *vary);
+
 /* cache_vcl.c */
 void VCL_Init(void);
 void VCL_Refresh(struct VCL_conf **vcc);
 void VCL_Rel(struct VCL_conf **vcc);
 void VCL_Get(struct VCL_conf **vcc);
 
+/* cache_lru.c */
+// void LRU_Init(void);
+void LRU_Enter(struct object *o, double stamp);
+void LRU_Remove(struct object *o);
+int LRU_DiscardOne(void);
+int LRU_DiscardSpace(int64_t quota);
+int LRU_DiscardTime(double cutoff);
+
 #define VCL_RET_MAC(l,u,b,n)
 #define VCL_MET_MAC(l,u,b) void VCL_##l##_method(struct sess *);
 #include "vcl_returns.h"
