@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006 Linpro AS
+ * Copyright (c) 2006-2007 Linpro AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -14,7 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  *
- * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
+ * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
@@ -26,7 +26,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: cache_ban.c 1183 2006-10-18 14:27:13Z des $
+ * $Id: cache_ban.c 2034 2007-09-25 08:54:11Z des $
  *
  * Ban processing
  */
@@ -42,17 +42,17 @@
 #include "cache.h"
 
 struct ban {
-	TAILQ_ENTRY(ban)	list;
+	VTAILQ_ENTRY(ban)	list;
 	unsigned		gen;
 	regex_t			regexp;
 	char			*ban;
 };
 
-static TAILQ_HEAD(,ban) ban_head = TAILQ_HEAD_INITIALIZER(ban_head);
+static VTAILQ_HEAD(,ban) ban_head = VTAILQ_HEAD_INITIALIZER(ban_head);
 static unsigned ban_next;
 static struct ban *ban_start;
 
-static void
+void
 AddBan(const char *regexp)
 {
 	struct ban *b;
@@ -70,7 +70,7 @@ AddBan(const char *regexp)
 	}
 	b->gen = ++ban_next;
 	b->ban = strdup(regexp);
-	TAILQ_INSERT_HEAD(&ban_head, b, list);
+	VTAILQ_INSERT_HEAD(&ban_head, b, list);
 	ban_start = b;
 }
 
@@ -90,7 +90,7 @@ BAN_CheckObject(struct object *o, const char *url)
 	b0 = ban_start;
 	for (b = b0;
 	    b != NULL && b->gen > o->ban_seq;
-	    b = TAILQ_NEXT(b, list)) {
+	    b = VTAILQ_NEXT(b, list)) {
 		i = regexec(&b->regexp, url, 0, NULL, 0);
 		if (!i)
 			return (1);
