@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006 Linpro AS
+ * Copyright (c) 2006-2007 Linpro AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -14,7 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  *
- * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
+ * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
@@ -26,7 +26,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: cache_hash.c 1096 2006-09-20 07:50:27Z des $
+ * $Id: cache_hash.c 2034 2007-09-25 08:54:11Z des $
  *
  * This is the central hash-table code, it relies on a chosen hash
  * implementation only for the actual hashing, all the housekeeping
@@ -64,28 +64,20 @@
 
 static struct hash_slinger      *hash;
 
-struct object *
-HSH_Lookup(struct sess *sp)
+/* Precreate an objhead and object for later use */
+void
+HSH_Prealloc(struct sess *sp)
 {
 	struct worker *w;
-	struct http *h;
-	struct objhead *oh;
-	struct object *o;
-	char *url, *host;
 
 	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
-	CHECK_OBJ_NOTNULL(sp->wrk, WORKER_MAGIC);
-	CHECK_OBJ_NOTNULL(sp->http, HTTP_MAGIC);
-	AN(hash);
 	w = sp->wrk;
-	h = sp->http;
 
-	/* Precreate an objhead and object in case we need them */
 	if (w->nobjhead == NULL) {
 		w->nobjhead = calloc(sizeof *w->nobjhead, 1);
 		XXXAN(w->nobjhead);
 		w->nobjhead->magic = OBJHEAD_MAGIC;
-		TAILQ_INIT(&w->nobjhead->objects);
+		VTAILQ_INIT(&w->nobjhead->objects);
 		MTX_INIT(&w->nobjhead->mtx);
 		VSL_stats->n_objecthead++;
 	} else
@@ -97,15 +89,82 @@ HSH_Lookup(struct sess *sp)
 		w->nobj->http.magic = HTTP_MAGIC;
 		w->nobj->busy = 1;
 		w->nobj->refcnt = 1;
-		TAILQ_INIT(&w->nobj->store);
-		TAILQ_INIT(&w->nobj->waitinglist);
+		VTAILQ_INIT(&w->nobj->store);
+		VTAILQ_INIT(&w->nobj->waitinglist);
 		VSL_stats->n_object++;
 	} else
 		CHECK_OBJ_NOTNULL(w->nobj, OBJECT_MAGIC);
+}
 
-	url = h->hd[HTTP_HDR_URL].b;
-	if (!http_GetHdr(h, H_Host, &host))
-		host = url;
+static void
+HSH_Freestore(struct object *o)
+{
+	struct storage *st, *stn;
+
+	VTAILQ_FOREACH_SAFE(st, &o->store, list, stn) {
+		CHECK_OBJ_NOTNULL(st, STORAGE_MAGIC);
+		VTAILQ_REMOVE(&o->store, st, list);
+		STV_free(st);
+	}
+}
+
+int
+HSH_Compare(struct sess *sp, const char *b, const char *e)
+{
+	int i;
+	unsigned u, v;
+
+	i = sp->lhashptr - (e - b);
+	if (i)
+		return (i);
+	for (u = 0; u < sp->ihashptr; u += 2) {
+		v = sp->hashptr[u + 1] - sp->hashptr[u];
+		i = memcmp(sp->hashptr[u], b, v);
+		if (i)
+			return (i);
+		b += v;
+		i = '#' - *b++;
+		if (i)
+			return (i);
+	}
+	assert(*b == '\0');
+	b++;
+	assert(b == e);
+	return (0);
+}
+
+void
+HSH_Copy(struct sess *sp, char *b, const char *e)
+{
+	unsigned u, v;
+
+	assert((e - b) >= sp->lhashptr);
+	for (u = 0; u < sp->ihashptr; u += 2) {
+		v = sp->hashptr[u + 1] - sp->hashptr[u];
+		memcpy(b, sp->hashptr[u], v);
+		b += v;
+		*b++ = '#';
+	}
+	*b++ = '\0';
+	assert(b <= e);
+}
+
+struct object *
+HSH_Lookup(struct sess *sp)
+{
+	struct worker *w;
+	struct http *h;
+	struct objhead *oh;
+	struct object *o;
+
+	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
+	CHECK_OBJ_NOTNULL(sp->wrk, WORKER_MAGIC);
+	CHECK_OBJ_NOTNULL(sp->http, HTTP_MAGIC);
+	AN(hash);
+	w = sp->wrk;
+	h = sp->http;
+
+	HSH_Prealloc(sp);
 	if (sp->obj != NULL) {
 		CHECK_OBJ_NOTNULL(sp->obj, OBJECT_MAGIC);
 		o = sp->obj;
@@ -114,38 +173,40 @@ HSH_Lookup(struct sess *sp)
 		LOCK(&oh->mtx);
 		goto were_back;
 	}
-	oh = hash->lookup(url, host, w->nobjhead);
+
+	oh = hash->lookup(sp, w->nobjhead);
 	CHECK_OBJ_NOTNULL(oh, OBJHEAD_MAGIC);
 	if (oh == w->nobjhead)
 		w->nobjhead = NULL;
 	LOCK(&oh->mtx);
-	TAILQ_FOREACH(o, &oh->objects, list) {
+	VTAILQ_FOREACH(o, &oh->objects, list) {
 		o->refcnt++;
 		if (o->busy) {
-			TAILQ_INSERT_TAIL(&o->waitinglist, sp, list);
+			VTAILQ_INSERT_TAIL(&o->waitinglist, sp, list);
 			sp->obj = o;
 			UNLOCK(&oh->mtx);
 			return (NULL);
 		}
 	were_back:
-		/* XXX: check Vary: */
 		if (!o->cacheable) {
 			/* ignore */
 		} else if (o->ttl == 0) {
 			/* Object banned but not reaped yet */
-		} else if (o->ttl <= sp->t_req.tv_sec) {
+		} else if (o->ttl <= sp->t_req) {
 			/* Object expired */
-		} else if (BAN_CheckObject(o, url)) {
+		} else if (BAN_CheckObject(o, h->hd[HTTP_HDR_URL].b)) {
 			o->ttl = 0;
 			VSL(SLT_ExpBan, 0, "%u was banned", o->xid);
-			EXP_TTLchange(o);
-		} else
+			if (o->heap_idx != 0)
+				EXP_TTLchange(o);
+		} else if (o->vary == NULL || VRY_Match(sp, o->vary))
 			break;
 		o->refcnt--;
 	}
 	if (o != NULL) {
 		UNLOCK(&oh->mtx);
 		(void)hash->deref(oh);
+		LRU_Enter(o, sp->t_req);
 		return (o);
 	}
 
@@ -153,16 +214,18 @@ HSH_Lookup(struct sess *sp)
 	o = w->nobj;
 	w->nobj = NULL;
 	o->objhead = oh;
-	TAILQ_INSERT_TAIL(&oh->objects, o, list);
+	VTAILQ_INSERT_TAIL(&oh->objects, o, list);
 	/* NB: do not deref objhead the new object inherits our reference */
 	UNLOCK(&oh->mtx);
 	BAN_NewObj(o);
+	LRU_Enter(o, sp->t_req);
 	return (o);
 }
 
 void
 HSH_Unbusy(struct object *o)
 {
+	struct objhead *oh;
 	struct sess *sp;
 
 	CHECK_OBJ_NOTNULL(o, OBJECT_MAGIC);
@@ -170,14 +233,19 @@ HSH_Unbusy(struct object *o)
 	assert(o->refcnt > 0);
 	if (o->cacheable)
 		EXP_Insert(o);
-	LOCK(&o->objhead->mtx);
+	oh = o->objhead;
+	if (oh != NULL) {
+		CHECK_OBJ(oh, OBJHEAD_MAGIC);
+		LOCK(&oh->mtx);
+	}
 	o->busy = 0;
-	UNLOCK(&o->objhead->mtx);
+	if (oh != NULL)
+		UNLOCK(&oh->mtx);
 	while (1) {
-		sp = TAILQ_FIRST(&o->waitinglist);
+		sp = VTAILQ_FIRST(&o->waitinglist);
 		if (sp == NULL)
 			break;
-		TAILQ_REMOVE(&o->waitinglist, sp, list);
+		VTAILQ_REMOVE(&o->waitinglist, sp, list);
 		WRK_QueueSession(sp);
 	}
 }
@@ -189,55 +257,62 @@ HSH_Ref(struct object *o)
 
 	CHECK_OBJ_NOTNULL(o, OBJECT_MAGIC);
 	oh = o->objhead;
-	CHECK_OBJ_NOTNULL(oh, OBJHEAD_MAGIC);
-	LOCK(&oh->mtx);
+	if (oh != NULL) {
+		CHECK_OBJ(oh, OBJHEAD_MAGIC);
+		LOCK(&oh->mtx);
+	}
 	assert(o->refcnt > 0);
 	o->refcnt++;
-	UNLOCK(&oh->mtx);
+	if (oh != NULL)
+		UNLOCK(&oh->mtx);
 }
 
 void
 HSH_Deref(struct object *o)
 {
 	struct objhead *oh;
-	struct storage *st, *stn;
 	unsigned r;
 
 	CHECK_OBJ_NOTNULL(o, OBJECT_MAGIC);
 	oh = o->objhead;
-	CHECK_OBJ_NOTNULL(oh, OBJHEAD_MAGIC);
+	if (oh != NULL) {
+		CHECK_OBJ(oh, OBJHEAD_MAGIC);
 
-	/* drop ref on object */
-	LOCK(&oh->mtx);
+		/* drop ref on object */
+		LOCK(&oh->mtx);
+	}
 	assert(o->refcnt > 0);
 	r = --o->refcnt;
-	if (!r)
-		TAILQ_REMOVE(&oh->objects, o, list);
-	UNLOCK(&oh->mtx);
+	if (oh != NULL) {
+		if (!r)
+			VTAILQ_REMOVE(&oh->objects, o, list);
+		UNLOCK(&oh->mtx);
+	}
 
 	/* If still referenced, done */
 	if (r != 0)
 		return;
 
-	if (o->http.s != NULL) {
-		free(o->http.s);
-	}
+	if (o->http.ws->s != NULL)
+		free(o->http.ws->s);
 
-	TAILQ_FOREACH_SAFE(st, &o->store, list, stn) {
-		CHECK_OBJ_NOTNULL(st, STORAGE_MAGIC);
-		TAILQ_REMOVE(&o->store, st, list);
-		st->stevedore->free(st);
-	}
-	free(o);
+	if (o->vary != NULL)
+		free(o->vary);
+
+	LRU_Remove(o);
+	HSH_Freestore(o);
+	FREE_OBJ(o);
 	VSL_stats->n_object--;
 
+	if (oh == NULL)
+		return;
 	/* Drop our ref on the objhead */
 	if (hash->deref(oh))
 		return;
-	assert(TAILQ_EMPTY(&oh->objects));
+	assert(VTAILQ_EMPTY(&oh->objects));
 	MTX_DESTROY(&oh->mtx);
 	VSL_stats->n_objecthead--;
-	free(oh);
+	FREE_OBJ(oh);
 }
 
 void
