@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006 Linpro AS
+ * Copyright (c) 2006-2007 Linpro AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -14,7 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  *
- * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
+ * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
@@ -26,143 +26,42 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: cache_response.c 1183 2006-10-18 14:27:13Z des $
+ * $Id: cache_response.c 2034 2007-09-25 08:54:11Z des $
  */
 
-#include <stdio.h>		/* XXX: for NULL ?? */
-#include <string.h>		/* XXX: for NULL ?? */
 #include <sys/types.h>
 #include <sys/time.h>
 
-#ifndef HAVE_CLOCK_GETTIME
-#include "compat/clock_gettime.h"
-#endif
+#include <stdio.h>
+#include <stdlib.h>
 
 #include "shmlog.h"
 #include "heritage.h"
 #include "cache.h"
 
-/*--------------------------------------------------------------------*/
-/* List of canonical HTTP response code names from RFC2616 */
-
-static struct http_msg {
-	unsigned	nbr;
-	const char	*txt;
-	const char	*expl;
-} http_msg[] = {
-	{ 101, "Switching Protocols" },
-	{ 200, "OK" },
-	{ 201, "Created" },
-	{ 202, "Accepted" },
-	{ 203, "Non-Authoritative Information" },
-	{ 204, "No Content" },
-	{ 205, "Reset Content" },
-	{ 206, "Partial Content" },
-	{ 300, "Multiple Choices" },
-	{ 301, "Moved Permanently" },
-	{ 302, "Found" },
-	{ 303, "See Other" },
-	{ 304, "Not Modified" },
-	{ 305, "Use Proxy" },
-	{ 306, "(Unused)" },
-	{ 307, "Temporary Redirect" },
-	{ 400, "Bad Request" },
-	{ 401, "Unauthorized" },
-	{ 402, "Payment Required" },
-	{ 403, "Forbidden" },
-	{ 404, "Not Found" },
-	{ 405, "Method Not Allowed" },
-	{ 406, "Not Acceptable" },
-	{ 407, "Proxy Authentication Required" },
-	{ 408, "Request Timeout" },
-	{ 409, "Conflict" },
-	{ 410, "Gone" },
-	{ 411, "Length Required" },
-	{ 412, "Precondition Failed" },
-	{ 413, "Request Entity Too Large" },
-	{ 414, "Request-URI Too Long" },
-	{ 415, "Unsupported Media Type" },
-	{ 416, "Requested Range Not Satisfiable" },
-	{ 417, "Expectation Failed" },
-	{ 500, "Internal Server Error" },
-	{ 501, "Not Implemented" },
-	{ 502, "Bad Gateway" },
-	{ 503, "Service Unavailable" },
-	{ 504, "Gateway Timeout" },
-	{ 505, "HTTP Version Not Supported" },
-	{ 0, NULL }
-};
-
 /*--------------------------------------------------------------------*/
 
 void
-RES_Error(struct sess *sp, int code, const char *expl)
+RES_Error(struct sess *sp, int code, const char *reason)
 {
-	char buf[40];
-	struct vsb *sb;
-	struct http_msg *mp;
-	const char *msg;
 
-	assert(code >= 100 && code <= 999);
-	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
+	/* get a pristine object */
+	HSH_Prealloc(sp);
+	sp->obj = sp->wrk->nobj;
+	sp->wrk->nobj = NULL;
+	sp->obj->busy = 1;
 
-	clock_gettime(CLOCK_REALTIME, &sp->t_resp);
+	/* synthesize error page and send it */
+	SYN_ErrorPage(sp, code, reason, 0);
+	RES_BuildHttp(sp);
+	RES_WriteObj(sp);
 
-	msg = "Unknown error";
-	for (mp = http_msg; mp->nbr != 0 && mp->nbr <= code; mp++)  {
-		if (mp->nbr < code)
-			continue;
-		if (mp->nbr > code)
-			break;
-		msg = mp->txt;
-		if (expl == NULL)
-			expl = mp->expl;
-		break;
-	}
-	if (expl == NULL)
-		expl = msg;
-	AN(expl);
-	AN(msg);
-
-	sb = vsb_new(NULL, NULL, 0, VSB_AUTOEXTEND);
-	XXXAN(sb);
-
-	vsb_clear(sb);
-	vsb_printf(sb, "HTTP/1.1 %03d %s\r\n", code, msg);
-	TIM_format(sp->t_req.tv_sec, buf);
-	vsb_printf(sb, "Date: %s\r\n", buf);
-	vsb_cat(sb,
-		"Server: Varnish\r\n"
-		"Connection: close\r\n"
-		"Content-Type: text/html; charset=iso-8859-1\r\n"
-		"\r\n"
-		"<!DOCTYPE HTML PUBLIC \"-//IETF//DTD HTML 2.0//EN\">\r\n"
-		"<HTML>\r\n"
-		"  <HEAD>\r\n");
-	vsb_printf(sb, "    <TITLE>%03d %s</TITLE>\r\n", code, msg);
-	vsb_cat(sb,
-		"  </HEAD>\r\n"
-		"  <BODY>\r\n");
-	vsb_printf(sb, "    <H1>Error %03d %s</H1>\r\n", code, msg);
-	vsb_printf(sb, "    <P>%s</P>\r\n", expl);
-	vsb_printf(sb, "    <H3>Guru Meditation:</H3>\r\n", expl);
-	vsb_printf(sb, "    <P>XID: %u</P>\r\n", sp->xid);
-	vsb_cat(sb,
-		"    <I><A href=\"http://varnish.linpro.no/\">Varnish</A></I>\r\n"
-		"  </BODY>\r\n"
-		"</HTML>\r\n");
-	vsb_finish(sb);
-	WRK_Reset(sp->wrk, &sp->fd);
-	sp->wrk->acct.hdrbytes += WRK_Write(sp->wrk, vsb_data(sb), vsb_len(sb));
-	WRK_Flush(sp->wrk);
-	WSL(sp->wrk, SLT_TxStatus, sp->id, "%d", code);
-	WSL(sp->wrk, SLT_TxProtocol, sp->id, "HTTP/1.1");
-	WSL(sp->wrk, SLT_TxResponse, sp->id, msg);
-	vca_close_session(sp, expl);
-	vsb_delete(sb);
+	/* GC the error page */
+	HSH_Unbusy(sp->obj);
+	HSH_Deref(sp->obj);
+	sp->obj = NULL;
 }
 
-
 /*--------------------------------------------------------------------*/
 
 static void
@@ -174,20 +73,16 @@ res_do_304(struct sess *sp)
 
 	http_ClrHeader(sp->http);
 	sp->http->logtag = HTTP_Tx;
-	http_SetResp(sp->wrk, sp->fd, sp->http,
-	    "HTTP/1.1", "304", "Not Modified");
-	TIM_format(sp->t_req.tv_sec, lm);
+	http_SetResp(sp->http, "HTTP/1.1", "304", "Not Modified");
+	TIM_format(sp->t_req, lm);
 	http_PrintfHeader(sp->wrk, sp->fd, sp->http, "Date: %s", lm);
 	http_SetHeader(sp->wrk, sp->fd, sp->http, "Via: 1.1 varnish");
 	http_PrintfHeader(sp->wrk, sp->fd, sp->http, "X-Varnish: %u", sp->xid);
 	TIM_format(sp->obj->last_modified, lm);
 	http_PrintfHeader(sp->wrk, sp->fd, sp->http, "Last-Modified: %s", lm);
-	if (sp->doclose != NULL)
-		http_SetHeader(sp->wrk, sp->fd, sp->http, "Connection: close");
-	WRK_Reset(sp->wrk, &sp->fd);
-	sp->wrk->acct.hdrbytes += http_Write(sp->wrk, sp->http, 1);
-	if (WRK_Flush(sp->wrk))
-		vca_close_session(sp, "remote closed");
+	http_PrintfHeader(sp->wrk, sp->fd, sp->http, "Connection: %s",
+	    sp->doclose ? "close" : "keep-alive");
+	sp->wantbody = 0;
 }
 
 /*--------------------------------------------------------------------*/
@@ -196,12 +91,12 @@ static int
 res_do_conds(struct sess *sp)
 {
 	char *p;
-	time_t ims;
+	double ims;
 
 	if (sp->obj->last_modified > 0 &&
 	    http_GetHdr(sp->http, H_If_Modified_Since, &p)) {
 		ims = TIM_parse(p);
-		if (ims > sp->t_req.tv_sec)	/* [RFC2616 14.25] */
+		if (ims > sp->t_req)	/* [RFC2616 14.25] */
 			return (0);
 		if (sp->obj->last_modified > ims) {
 			return (0);
@@ -215,13 +110,11 @@ res_do_conds(struct sess *sp)
 /*--------------------------------------------------------------------*/
 
 void
-RES_WriteObj(struct sess *sp)
+RES_BuildHttp(struct sess *sp)
 {
-	struct storage *st;
-	unsigned u = 0;
+	char time_str[30];
 
 	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
-	clock_gettime(CLOCK_REALTIME, &sp->t_resp);
 
 	if (sp->obj->response == 200 && sp->http->conds && res_do_conds(sp))
 		return;
@@ -230,24 +123,42 @@ RES_WriteObj(struct sess *sp)
 
 	http_ClrHeader(sp->http);
 	sp->http->logtag = HTTP_Tx;
-	http_CopyResp(sp->wrk, sp->fd, sp->http, &sp->obj->http);
-	http_FilterHeader(sp->wrk, sp->fd, sp->http, &sp->obj->http, HTTPH_A_DELIVER);
+	http_CopyResp(sp->http, &sp->obj->http);
+	http_FilterFields(sp->wrk, sp->fd, sp->http, &sp->obj->http,
+	    HTTPH_A_DELIVER);
+	
+	TIM_format(TIM_real(), time_str);
+	http_PrintfHeader(sp->wrk, sp->fd, sp->http, "Date: %s", time_str);
+	
 	if (sp->xid != sp->obj->xid)
 		http_PrintfHeader(sp->wrk, sp->fd, sp->http,
 		    "X-Varnish: %u %u", sp->xid, sp->obj->xid);
 	else
-		http_PrintfHeader(sp->wrk, sp->fd, sp->http, "X-Varnish: %u", sp->xid);
-	http_PrintfHeader(sp->wrk, sp->fd, sp->http, "Age: %u",
-	    sp->obj->age + sp->t_resp.tv_sec - sp->obj->entered);
+		http_PrintfHeader(sp->wrk, sp->fd, sp->http,
+		    "X-Varnish: %u", sp->xid);
+	http_PrintfHeader(sp->wrk, sp->fd, sp->http, "Age: %.0f",
+	    sp->obj->age + sp->t_resp - sp->obj->entered);
 	http_SetHeader(sp->wrk, sp->fd, sp->http, "Via: 1.1 varnish");
-	if (sp->doclose != NULL)
-		http_SetHeader(sp->wrk, sp->fd, sp->http, "Connection: close");
+	http_PrintfHeader(sp->wrk, sp->fd, sp->http, "Connection: %s",
+	    sp->doclose ? "close" : "keep-alive");
+}
+
+/*--------------------------------------------------------------------*/
+
+void
+RES_WriteObj(struct sess *sp)
+{
+	struct storage *st;
+	unsigned u = 0;
+
+	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
+
 	WRK_Reset(sp->wrk, &sp->fd);
 	sp->wrk->acct.hdrbytes += http_Write(sp->wrk, sp->http, 1);
 	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
 
 	if (sp->wantbody) {
-		TAILQ_FOREACH(st, &sp->obj->store, list) {
+		VTAILQ_FOREACH(st, &sp->obj->store, list) {
 			CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
 			CHECK_OBJ_NOTNULL(st, STORAGE_MAGIC);
 			AN(st->stevedore);
@@ -255,7 +166,7 @@ RES_WriteObj(struct sess *sp)
 			sp->wrk->acct.bodybytes += st->len;
 #ifdef HAVE_SENDFILE
 			/*
-			 * XXX: the overhead of setting up senddile is not
+			 * XXX: the overhead of setting up sendfile is not
 			 * XXX: epsilon and maybe not even delta, so avoid
 			 * XXX: engaging sendfile for small objects.
 			 * XXX: Should use getpagesize() ?
