@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006 Linpro AS
+ * Copyright (c) 2006-2007 Linpro AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -14,7 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  *
- * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
+ * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
@@ -26,7 +26,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: mgt_cli.c 1096 2006-09-20 07:50:27Z des $
+ * $Id: mgt_cli.c 2307 2007-12-18 17:20:00Z des $
  *
  * The management process' CLI handling
  */
@@ -54,9 +54,9 @@
 #include "mgt_event.h"
 #include "shmlog.h"
 
+#include "vss.h"
+
 static int		cli_i = -1, cli_o = -1;
-static int		telnet_sock;
-static struct ev	*telnet_ev;
 
 /*--------------------------------------------------------------------*/
 
@@ -68,8 +68,8 @@ mcf_stats(struct cli *cli, char **av, void *priv)
 	(void)priv;
 
 	AN(VSL_stats);
-#define MAC_STAT(n,t,f,d) \
-    cli_out(cli, "%12ju  " d "\n", (VSL_stats->n));
+#define MAC_STAT(n, t, f, d) \
+    cli_out(cli, "%12ju  %s\n", (VSL_stats->n), d);
 #include "stat_field.h"
 #undef MAC_STAT
 }
@@ -83,8 +83,9 @@ mcf_stats(struct cli *cli, char **av, void *priv)
 static void
 mcf_passthru(struct cli *cli, char **av, void *priv)
 {
-	char *p, *q, *r;
-	unsigned u, v;
+	struct vsb *sb;
+	char *p;
+	unsigned u;
 	int i;
 
 	(void)priv;
@@ -95,30 +96,35 @@ mcf_passthru(struct cli *cli, char **av, void *priv)
 		cli_out(cli, "Cache process not running");
 		return;
 	}
-	v = 0;
-	for (u = 1; av[u] != NULL; u++)
-		v += strlen(av[u]) + 3;
-	p = malloc(v);
-	XXXAN(p);
-	q = p;
+	sb = vsb_new(NULL, NULL, 64, VSB_AUTOEXTEND);
+	XXXAN(sb);
 	for (u = 1; av[u] != NULL; u++) {
-		*q++ = '"';
-		for (r = av[u]; *r; r++) {
-			switch (*r) {
-			case '\\':	*q++ = '\\'; *q++ = '\\'; break;
-			case '\n':	*q++ = '\\'; *q++ = 'n'; break;
-			case '"':	*q++ = '\\'; *q++ = '"'; break;
-			default:	*q++ = *r; break;
+		if (u > 1)
+			vsb_putc(sb, ' ');
+		vsb_putc(sb, '"');
+		for (p = av[u]; *p; p++) {
+			switch (*p) {
+			case '\\':
+				vsb_cat(sb, "\\\\");
+				break;
+			case '\n':
+				vsb_cat(sb, "\\n");
+				break;
+			case '"':
+				vsb_cat(sb, "\\\"");
+				break;
+			default:
+				vsb_putc(sb, *p);
 			}
 		}
-		*q++ = '"';
-		*q++ = ' ';
+		vsb_putc(sb, '"');
 	}
-	*q++ = '\n';
-	v = q - p;
-	i = write(cli_o, p, v);
-	xxxassert(i == v);
-	free(p);
+	vsb_putc(sb, '\n');
+	xxxassert(!vsb_overflowed(sb));
+	vsb_finish(sb);
+	i = write(cli_o, vsb_data(sb), vsb_len(sb));
+	xxxassert(i == vsb_len(sb));
+	vsb_delete(sb);
 
 	i = cli_readres(cli_i, &u, &p, 3.0);
 	cli_result(cli, u);
@@ -134,6 +140,7 @@ static struct cli_proto *cli_proto;
 /* XXX: what order should this list be in ? */
 static struct cli_proto mgt_cli_proto[] = {
 	{ CLI_PING,		cli_func_ping },
+	{ CLI_SERVER_STATUS,	mcf_server_status, NULL },
 	{ CLI_SERVER_START,	mcf_server_startstop, NULL },
 	{ CLI_SERVER_STOP,	mcf_server_startstop, &cli_proto },
 	{ CLI_STATS,		mcf_stats, NULL },
@@ -142,6 +149,7 @@ static struct cli_proto mgt_cli_proto[] = {
 	{ CLI_VCL_USE,		mcf_config_use, NULL },
 	{ CLI_VCL_DISCARD,	mcf_config_discard, NULL },
 	{ CLI_VCL_LIST,		mcf_config_list, NULL },
+	{ CLI_VCL_SHOW,		mcf_config_show, NULL },
 	{ CLI_PARAM_SHOW,	mcf_param_show, NULL },
 	{ CLI_PARAM_SET,	mcf_param_set, NULL },
 	{ CLI_HELP,		cli_func_help, NULL },
@@ -276,8 +284,8 @@ struct cli_port {
 	int			fdo;
 	int			verbose;
 	char			*buf;
-	unsigned		nbuf;
-	unsigned		lbuf;
+	int			nbuf; /* next free position in buf */
+	int			lbuf; /* length of buf */
 	struct cli		cli[1];
 	char			name[30];
 };
@@ -286,39 +294,54 @@ static int
 mgt_cli_callback(struct ev *e, int what)
 {
 	struct cli_port *cp;
-	char *p;
+	char *p, *q;
 	int i;
 
 	CAST_OBJ_NOTNULL(cp, e->priv, CLI_PORT_MAGIC);
 
-	while (!(what & (EV_ERR | EV_HUP))) {
-		if (cp->nbuf == cp->lbuf) {
-			cp->lbuf += cp->lbuf;
-			cp->buf = realloc(cp->buf, cp->lbuf);
-			XXXAN(cp->buf);
-		}
-		i = read(cp->fdi, cp->buf + cp->nbuf, cp->lbuf - cp->nbuf);
-		if (i <= 0)
-			break;
-		cp->nbuf += i;
-		p = strchr(cp->buf, '\n');
-		if (p == NULL)
-			return (0);
-		*p = '\0';
-		fprintf(stderr, "CLI <%s>\n", cp->buf);
-		vsb_clear(cp->cli->sb);
-		cli_dispatch(cp->cli, cli_proto, cp->buf);
-		vsb_finish(cp->cli->sb);
-		/* XXX: cp->verbose */
-		if (cli_writeres(cp->fdo, cp->cli))
-			break;
-		i = ++p - cp->buf;
-		assert(i <= cp->nbuf);
-		if (i < cp->nbuf)
-			memcpy(cp->buf, p, cp->nbuf - i);
-		cp->nbuf -= i;
-		return (0);
+	if (what & (EV_ERR | EV_HUP | EV_GONE))
+		goto cli_close;
+
+	/* grow the buffer if it is full */
+	if (cp->nbuf == cp->lbuf) {
+		cp->lbuf += cp->lbuf;
+		cp->buf = realloc(cp->buf, cp->lbuf);
+		XXXAN(cp->buf);
 	}
+
+	/* read more data into the buffer */
+	i = read(cp->fdi, cp->buf + cp->nbuf, cp->lbuf - cp->nbuf);
+	if (i <= 0)
+		goto cli_close;
+	cp->nbuf += i;
+
+	for (p = q = cp->buf; q < cp->buf + cp->nbuf; ++q) {
+		if (*q != '\n')
+			continue;
+		/* got a newline == got a command */
+		*q = '\0';
+		vsb_clear(cp->cli->sb);
+		cli_dispatch(cp->cli, cli_proto, p);
+		vsb_finish(cp->cli->sb);
+
+		/* send the result back */
+		if (cli_writeres(cp->fdo, cp->cli))
+			goto cli_close;
+
+		/* ready for next command */
+		p = q + 1;
+	}
+
+	/* see if there's any data left in the buffer */
+	if (p != q) {
+		assert(q == cp->buf + cp->nbuf);
+		cp->nbuf -= (p - cp->buf);
+		memmove(cp->buf, p, cp->nbuf);
+	} else
+		cp->nbuf = 0;
+	return (0);
+
+cli_close:
 	vsb_delete(cp->cli->sb);
 	free(cp->buf);
 	close(cp->fdi);
@@ -368,14 +391,13 @@ mgt_cli_setup(int fdi, int fdo, int verbose)
 static int
 telnet_accept(struct ev *ev, int what)
 {
-	socklen_t l;
-	struct sockaddr addr[2];	/* XXX IPv6 hack */
+	struct sockaddr_storage addr;
+	socklen_t addrlen;
 	int i;
 
-	(void)ev;
 	(void)what;
-	l = sizeof addr;
-	i = accept(telnet_sock, addr, &l);
+	addrlen = sizeof addr;
+	i = accept(ev->fd, (void *)&addr, &addrlen);
 	if (i < 0)
 		return (0);
 
@@ -386,21 +408,29 @@ telnet_accept(struct ev *ev, int what)
 int
 mgt_cli_telnet(const char *T_arg)
 {
+	struct vss_addr **ta;
 	char *addr, *port;
+	int i, n;
 
-	TCP_parse(T_arg, &addr, &port);
-	telnet_sock = TCP_open(addr, port, 0);
+	XXXAZ(VSS_parse(T_arg, &addr, &port));
+	XXXAN(n = VSS_resolve(addr, port, &ta));
 	free(addr);
 	free(port);
-	if (telnet_sock < 0) {
-		fprintf(stderr, "Could not open TELNET port\n");
-		exit (2);
+	if (n == 0) {
+		fprintf(stderr, "Could not open management port\n");
+		exit(2);
 	}
-	telnet_ev = ev_new();
-	XXXAN(telnet_ev);
-	telnet_ev->fd = telnet_sock;
-	telnet_ev->fd_flags = POLLIN;
-	telnet_ev->callback = telnet_accept;
-	ev_add(mgt_evb, telnet_ev);
+	for (i = 0; i < n; ++i) {
+		int sock = VSS_listen(ta[i], 1);
+		struct ev *ev = ev_new();
+		XXXAN(ev);
+		ev->fd = sock;
+		ev->fd_flags = POLLIN;
+		ev->callback = telnet_accept;
+		ev_add(mgt_evb, ev);
+		free(ta[i]);
+		ta[i] = NULL;
+	}
+	free(ta);
 	return (0);
 }
