@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006 Linpro AS
+ * Copyright (c) 2006-2007 Linpro AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -14,7 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  *
- * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
+ * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
@@ -26,14 +26,20 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: mgt_param.c 1167 2006-10-18 14:26:52Z des $
+ * $Id: mgt_param.c 2034 2007-09-25 08:54:11Z des $
  */
 
-#include <stdio.h>
-#include <string.h>
-#include <stdlib.h>
-#include <unistd.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+
+#include <grp.h>
 #include <limits.h>
+#include <pwd.h>
+#include <regex.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <unistd.h>
 
 #include "cli.h"
 #include "cli_priv.h"
@@ -43,6 +49,10 @@
 
 #include "heritage.h"
 
+#include "vss.h"
+
+#define MAGIC_INIT_STRING	"\001"
+
 struct parspec;
 
 typedef void tweak_t(struct cli *, struct parspec *, const char *arg);
@@ -50,15 +60,17 @@ typedef void tweak_t(struct cli *, struct parspec *, const char *arg);
 struct parspec {
 	const char	*name;
 	tweak_t		*func;
-	const char	*expl;
+	const char	*descr;
 	const char	*def;
 	const char	*units;
 };
 
+static struct params master;
+
 /*--------------------------------------------------------------------*/
 
 static void
-tweak_generic_timeout(struct cli *cli, unsigned *dst, const char *arg)
+tweak_generic_timeout(struct cli *cli, volatile unsigned *dst, const char *arg)
 {
 	unsigned u;
 
@@ -77,7 +89,7 @@ tweak_generic_timeout(struct cli *cli, unsigned *dst, const char *arg)
 /*--------------------------------------------------------------------*/
 
 static void
-tweak_generic_bool(struct cli *cli, unsigned *dest, const char *arg)
+tweak_generic_bool(struct cli *cli, volatile unsigned *dest, const char *arg)
 {
 	if (arg != NULL) {
 		if (!strcasecmp(arg, "off"))
@@ -104,7 +116,7 @@ tweak_generic_bool(struct cli *cli, unsigned *dest, const char *arg)
 /*--------------------------------------------------------------------*/
 
 static void
-tweak_generic_uint(struct cli *cli, unsigned *dest, const char *arg, unsigned min, unsigned max)
+tweak_generic_uint(struct cli *cli, volatile unsigned *dest, const char *arg, unsigned min, unsigned max)
 {
 	unsigned u;
 
@@ -131,6 +143,86 @@ tweak_generic_uint(struct cli *cli, unsigned *dest, const char *arg, unsigned mi
 	}
 }
 
+/*--------------------------------------------------------------------
+ * XXX: slightly magic.  We want to initialize to "nobody" (XXX: shouldn't
+ * XXX: that be something autocrap found for us ?) but we don't want to
+ * XXX: fail initialization if that user doesn't exists, even though we
+ * XXX: do want to fail it, in subsequent sets.
+ * XXX: The magic init string is a hack for this.
+ */
+
+static void
+tweak_user(struct cli *cli, struct parspec *par, const char *arg)
+{
+	struct passwd *pw;
+	struct group *gr;
+
+	(void)par;
+	if (arg != NULL) {
+		if (!strcmp(arg, MAGIC_INIT_STRING)) {
+			pw = getpwnam("nobody");
+			if (pw == NULL) {
+				master.uid = getuid();
+				return;
+			}
+		} else 
+			pw = getpwnam(arg);
+		if (pw == NULL) {
+			cli_out(cli, "Unknown user");
+			cli_result(cli, CLIS_PARAM);
+			return;
+		}
+		REPLACE(master.user, pw->pw_name);
+		master.uid = pw->pw_uid;
+		master.gid = pw->pw_gid;
+
+		/* set group to user's primary group */
+		if ((gr = getgrgid(pw->pw_gid)) != NULL &&
+		    (gr = getgrnam(gr->gr_name)) != NULL &&
+		    gr->gr_gid == pw->pw_gid) 
+			REPLACE(master.group, gr->gr_name);
+	} else if (master.user) {
+		cli_out(cli, "%s (%d)", master.user, (int)master.uid);
+	} else {
+		cli_out(cli, "%d", (int)master.uid);
+	}
+}
+
+/*--------------------------------------------------------------------
+ * XXX: see comment for tweak_user, same thing here.
+ */
+
+static void
+tweak_group(struct cli *cli, struct parspec *par, const char *arg)
+{
+	struct group *gr;
+
+	(void)par;
+	if (arg != NULL) {
+		if (!strcmp(arg, MAGIC_INIT_STRING)) {
+			gr = getgrnam("nogroup");
+			if (gr == NULL) {
+				/* Only replace if tweak_user didn't */
+				if (master.gid == 0)
+					master.gid = getgid();
+				return;
+			}
+		} else
+			gr = getgrnam(arg);
+		if (gr == NULL) {
+			cli_out(cli, "Unknown group");
+			cli_result(cli, CLIS_PARAM);
+			return;
+		}
+		REPLACE(master.group, gr->gr_name);
+		master.gid = gr->gr_gid;
+	} else if (master.group) {
+		cli_out(cli, "%s (%d)", master.group, (int)master.gid);
+	} else {
+		cli_out(cli, "%d", (int)master.gid);
+	}
+}
+
 /*--------------------------------------------------------------------*/
 
 static void
@@ -138,7 +230,7 @@ tweak_default_ttl(struct cli *cli, struct parspec *par, const char *arg)
 {
 
 	(void)par;
-	tweak_generic_uint(cli, &params->default_ttl, arg, 0, UINT_MAX);
+	tweak_generic_uint(cli, &master.default_ttl, arg, 0, UINT_MAX);
 }
 
 /*--------------------------------------------------------------------*/
@@ -148,7 +240,7 @@ tweak_thread_pools(struct cli *cli, struct parspec *par, const char *arg)
 {
 
 	(void)par;
-	tweak_generic_uint(cli, &params->wthread_pools, arg,
+	tweak_generic_uint(cli, &master.wthread_pools, arg,
 	    1, UINT_MAX);
 }
 
@@ -160,8 +252,8 @@ tweak_thread_pool_min(struct cli *cli, struct parspec *par, const char *arg)
 {
 
 	(void)par;
-	tweak_generic_uint(cli, &params->wthread_min, arg,
-	    0, params->wthread_max);
+	tweak_generic_uint(cli, &master.wthread_min, arg,
+	    0, master.wthread_max);
 }
 
 /*--------------------------------------------------------------------*/
@@ -171,8 +263,8 @@ tweak_thread_pool_max(struct cli *cli, struct parspec *par, const char *arg)
 {
 
 	(void)par;
-	tweak_generic_uint(cli, &params->wthread_max, arg,
-	    params->wthread_min, UINT_MAX);
+	tweak_generic_uint(cli, &master.wthread_max, arg,
+	    master.wthread_min, UINT_MAX);
 }
 
 /*--------------------------------------------------------------------*/
@@ -182,7 +274,7 @@ tweak_thread_pool_timeout(struct cli *cli, struct parspec *par, const char *arg)
 {
 
 	(void)par;
-	tweak_generic_timeout(cli, &params->wthread_timeout, arg);
+	tweak_generic_timeout(cli, &master.wthread_timeout, arg);
 }
 
 /*--------------------------------------------------------------------*/
@@ -192,7 +284,7 @@ tweak_overflow_max(struct cli *cli, struct parspec *par, const char *arg)
 {
 
 	(void)par;
-	tweak_generic_uint(cli, &params->overflow_max, arg, 0, UINT_MAX);
+	tweak_generic_uint(cli, &master.overflow_max, arg, 0, UINT_MAX);
 }
 
 /*--------------------------------------------------------------------*/
@@ -202,7 +294,7 @@ tweak_http_workspace(struct cli *cli, struct parspec *par, const char *arg)
 {
 
 	(void)par;
-	tweak_generic_uint(cli, &params->mem_workspace, arg,
+	tweak_generic_uint(cli, &master.mem_workspace, arg,
 	    1024, UINT_MAX);
 }
 
@@ -212,7 +304,7 @@ static void
 tweak_sess_timeout(struct cli *cli, struct parspec *par, const char *arg)
 {
 	(void)par;
-	tweak_generic_timeout(cli, &params->sess_timeout, arg);
+	tweak_generic_timeout(cli, &master.sess_timeout, arg);
 }
 
 /*--------------------------------------------------------------------*/
@@ -221,7 +313,7 @@ static void
 tweak_pipe_timeout(struct cli *cli, struct parspec *par, const char *arg)
 {
 	(void)par;
-	tweak_generic_timeout(cli, &params->pipe_timeout, arg);
+	tweak_generic_timeout(cli, &master.pipe_timeout, arg);
 }
 
 /*--------------------------------------------------------------------*/
@@ -230,7 +322,7 @@ static void
 tweak_send_timeout(struct cli *cli, struct parspec *par, const char *arg)
 {
 	(void)par;
-	tweak_generic_timeout(cli, &params->send_timeout, arg);
+	tweak_generic_timeout(cli, &master.send_timeout, arg);
 }
 
 /*--------------------------------------------------------------------*/
@@ -240,7 +332,7 @@ tweak_auto_restart(struct cli *cli, struct parspec *par, const char *arg)
 {
 
 	(void)par;
-	tweak_generic_bool(cli, &params->auto_restart, arg);
+	tweak_generic_bool(cli, &master.auto_restart, arg);
 }
 
 /*--------------------------------------------------------------------*/
@@ -250,7 +342,7 @@ tweak_fetch_chunksize(struct cli *cli, struct parspec *par, const char *arg)
 {
 
 	(void)par;
-	tweak_generic_uint(cli, &params->fetch_chunksize, arg,
+	tweak_generic_uint(cli, &master.fetch_chunksize, arg,
 	    4, UINT_MAX / 1024);
 }
 
@@ -262,7 +354,7 @@ tweak_sendfile_threshold(struct cli *cli, struct parspec *par, const char *arg)
 {
 
 	(void)par;
-	tweak_generic_uint(cli, &params->sendfile_threshold, arg, 0, UINT_MAX);
+	tweak_generic_uint(cli, &master.sendfile_threshold, arg, 0, UINT_MAX);
 }
 #endif /* HAVE_SENDFILE */
 
@@ -272,39 +364,99 @@ static void
 tweak_vcl_trace(struct cli *cli, struct parspec *par, const char *arg)
 {
 	(void)par;
-	tweak_generic_bool(cli, &params->vcl_trace, arg);
+	tweak_generic_bool(cli, &master.vcl_trace, arg);
 }
 
 /*--------------------------------------------------------------------*/
 
+static void
+clean_listen_sock_head(struct listen_sock_head *lsh)
+{
+	struct listen_sock *ls, *ls2;
+
+	VTAILQ_FOREACH_SAFE(ls, lsh, list, ls2) {
+		VTAILQ_REMOVE(lsh, ls, list);
+		free(ls->addr);
+		free(ls);
+	}
+}
+
 static void
 tweak_listen_address(struct cli *cli, struct parspec *par, const char *arg)
 {
-	char *a, *p;
+	char **av;
+	int i;
+	struct listen_sock		*ls;
+	struct listen_sock_head		lsh;
 
 	(void)par;
-	if (arg != NULL) {
-		if (TCP_parse(arg, &a, &p) != 0) {
-			cli_out(cli, "Invalid listen address");
+	if (arg == NULL) {
+		/* Quote the string if we have more than one socket */
+		if (heritage.nsocks > 1)
+			cli_out(cli, "\"%s\"", master.listen_address);
+		else
+			cli_out(cli, "%s", master.listen_address);
+		return;
+	}
+
+	av = ParseArgv(arg, 0);
+	if (av[0] != NULL) {
+		cli_out(cli, "Parse error: %s", av[0]);
+		cli_result(cli, CLIS_PARAM);
+		FreeArgv(av);
+		return;
+	}
+	if (av[1] == NULL) {
+		cli_out(cli, "Empty listen address");
+		cli_result(cli, CLIS_PARAM);
+		FreeArgv(av);
+		return;
+	}
+	VTAILQ_INIT(&lsh);
+	for (i = 1; av[i] != NULL; i++) {
+		struct vss_addr **ta;
+		char *host, *port;
+		int j, n;
+
+		if (VSS_parse(av[i], &host, &port) != 0) {
+			cli_out(cli, "Invalid listen address \"%s\"", av[i]);
 			cli_result(cli, CLIS_PARAM);
-			return;
+			break;
 		}
-		if (p == NULL) {
-			p = strdup("http");
-			AN(p);
+		n = VSS_resolve(host, port ? port : "http", &ta);
+		free(host);
+		free(port);
+		if (n == 0) {
+			cli_out(cli, "Invalid listen address \"%s\"", av[i]);
+			cli_result(cli, CLIS_PARAM);
+			break;
 		}
-		TCP_check(cli, a, p);
-		if (cli->result != CLIS_OK)
-			return;
-		free(params->listen_address);
-		free(params->listen_host);
-		free(params->listen_port);
-		params->listen_address = strdup(arg);
-		AN(params->listen_address);
-		params->listen_host = a;
-		params->listen_port = p;
-	} else
-		cli_out(cli, "%s", params->listen_address);
+		for (j = 0; j < n; ++j) {
+			ls = calloc(sizeof *ls, 1);
+			AN(ls);
+			ls->sock = -1;
+			ls->addr = ta[j];
+			VTAILQ_INSERT_TAIL(&lsh, ls, list);
+		}
+		free(ta);
+	}
+	FreeArgv(av);
+	if (cli->result != CLIS_OK) {
+		clean_listen_sock_head(&lsh);
+		return;
+	}
+
+	REPLACE(master.listen_address, arg);
+
+	clean_listen_sock_head(&heritage.socks);
+	heritage.nsocks = 0;
+
+	while (!VTAILQ_EMPTY(&lsh)) {
+		ls = VTAILQ_FIRST(&lsh);
+		VTAILQ_REMOVE(&lsh, ls, list);
+		VTAILQ_INSERT_TAIL(&heritage.socks, ls, list);
+		heritage.nsocks++;
+	}
 }
 
 /*--------------------------------------------------------------------*/
@@ -313,7 +465,7 @@ static void
 tweak_listen_depth(struct cli *cli, struct parspec *par, const char *arg)
 {
 	(void)par;
-	tweak_generic_uint(cli, &params->listen_depth, arg, 0, UINT_MAX);
+	tweak_generic_uint(cli, &master.listen_depth, arg, 0, UINT_MAX);
 }
 
 /*--------------------------------------------------------------------*/
@@ -322,7 +474,7 @@ static void
 tweak_srcaddr_hash(struct cli *cli, struct parspec *par, const char *arg)
 {
 	(void)par;
-	tweak_generic_uint(cli, &params->srcaddr_hash, arg, 63, UINT_MAX);
+	tweak_generic_uint(cli, &master.srcaddr_hash, arg, 63, UINT_MAX);
 }
 
 /*--------------------------------------------------------------------*/
@@ -331,7 +483,7 @@ static void
 tweak_srcaddr_ttl(struct cli *cli, struct parspec *par, const char *arg)
 {
 	(void)par;
-	tweak_generic_uint(cli, &params->srcaddr_ttl, arg, 0, UINT_MAX);
+	tweak_generic_uint(cli, &master.srcaddr_ttl, arg, 0, UINT_MAX);
 }
 
 /*--------------------------------------------------------------------*/
@@ -340,7 +492,7 @@ static void
 tweak_backend_http11(struct cli *cli, struct parspec *par, const char *arg)
 {
 	(void)par;
-	tweak_generic_bool(cli, &params->backend_http11, arg);
+	tweak_generic_bool(cli, &master.backend_http11, arg);
 }
 
 /*--------------------------------------------------------------------*/
@@ -349,7 +501,16 @@ static void
 tweak_client_http11(struct cli *cli, struct parspec *par, const char *arg)
 {
 	(void)par;
-	tweak_generic_bool(cli, &params->client_http11, arg);
+	tweak_generic_bool(cli, &master.client_http11, arg);
+}
+
+/*--------------------------------------------------------------------*/
+
+static void
+tweak_ping_interval(struct cli *cli, struct parspec *par, const char *arg)
+{
+	(void)par;
+	tweak_generic_uint(cli, &master.ping_interval, arg, 0, UINT_MAX);
 }
 
 /*--------------------------------------------------------------------*/
@@ -370,12 +531,20 @@ tweak_client_http11(struct cli *cli, struct parspec *par, const char *arg)
 	"\nNB: We don't know yet if it is a good idea to change " \
 	"this parameter.  Caution advised.\n"
 
-
 /*
  * Remember to update varnishd.1 whenever you add / remove a parameter or
  * change its default value.
  */
 static struct parspec parspec[] = {
+	{ "user", tweak_user,
+		"The unprivileged user to run as.  Setting this will "
+		"also set \"group\" to the specified user's primary group.\n"
+		MUST_RESTART,
+		MAGIC_INIT_STRING },
+	{ "group", tweak_group,
+		"The unprivileged group to run as.\n"
+		MUST_RESTART,
+		MAGIC_INIT_STRING },
 	{ "default_ttl", tweak_default_ttl,
 		"The TTL assigned to objects if neither the backend nor "
 		"the VCL code assigns one.\n"
@@ -467,7 +636,7 @@ static struct parspec parspec[] = {
 		"may make sense to set this to \"unlimited\".\n"
 #endif
 		EXPERIMENTAL,
-		"8192", "bytes" },
+		"-1", "bytes" },
 #endif /* HAVE_SENDFILE */
 	{ "vcl_trace", tweak_vcl_trace,
 		"Trace VCL execution in the shmlog.\n"
@@ -477,9 +646,11 @@ static struct parspec parspec[] = {
 		"default. ",
 		"off", "bool" },
 	{ "listen_address", tweak_listen_address,
-		"The network address/port where Varnish services requests.\n"
+		"Whitespace separated list of network endpoints where "
+		"Varnish will accept requests.\n"
+		"Possible formats: host, host:port, :port\n"
 		MUST_RESTART,
-		"0.0.0.0:80" },
+		":80" },
 	{ "listen_depth", tweak_listen_depth,
 		"Listen(2) queue depth.\n"
 #if defined(__FreeBSD__)
@@ -511,6 +682,12 @@ static struct parspec parspec[] = {
 		"backend response."
 		EXPERIMENTAL,
 		"off", "bool" },
+	{ "ping_interval", tweak_ping_interval,
+		"Interval between pings from parent to child.\n"
+		"Zero will disable pinging entirely, which makes "
+		"it possible to attach a debugger to the child.\n"
+		MUST_RESTART,
+		"3", "seconds" },
 	{ NULL, NULL, NULL }
 };
 
@@ -547,7 +724,7 @@ mcf_param_show(struct cli *cli, char **av, void *priv)
 		if (av[2] != NULL) {
 			cli_out(cli, "%-20s Default is %s\n", "", pp->def);
 			/* Format text to 72 col width */
-			for (p = pp->expl; *p != '\0'; ) {
+			for (p = pp->descr; *p != '\0'; ) {
 				q = strchr(p, '\n');
 				if (q == NULL)
 					q = strchr(p, '\0');
@@ -576,6 +753,15 @@ mcf_param_show(struct cli *cli, char **av, void *priv)
 
 /*--------------------------------------------------------------------*/
 
+void
+MCF_ParamSync(void)
+{
+	if (params != &master)
+		*params = master;
+}
+
+/*--------------------------------------------------------------------*/
+
 void
 MCF_ParamSet(struct cli *cli, const char *param, const char *val)
 {
@@ -584,6 +770,7 @@ MCF_ParamSet(struct cli *cli, const char *param, const char *val)
 	for (pp = parspec; pp->name != NULL; pp++) {
 		if (!strcmp(pp->name, param)) {
 			pp->func(cli, pp, val);
+			MCF_ParamSync();
 			return;
 		}
 	}
@@ -615,4 +802,5 @@ MCF_ParamInit(struct cli *cli)
 		if (cli->result != CLIS_OK)
 			return;
 	}
+	params = &master;
 }
