@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006 Linpro AS
+ * Copyright (c) 2006-2007 Linpro AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -14,7 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  *
- * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
+ * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
@@ -26,17 +26,24 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: storage_file.c 1096 2006-09-20 07:50:27Z des $
+ * $Id: storage_file.c 2239 2007-11-08 21:30:05Z des $
  *
  * Storage method based on mmap'ed file
  */
 
 #include <sys/param.h>
 #include <sys/mman.h>
-#include <sys/mount.h>
 #include <sys/socket.h>
 #include <sys/stat.h>
 
+#ifdef HAVE_SYS_MOUNT_H
+#include <sys/mount.h>
+#endif
+
+#ifdef HAVE_SYS_STATVFS_H
+#include <sys/statvfs.h>
+#endif
+
 #ifdef HAVE_SYS_VFS_H
 #include <sys/vfs.h>
 #endif
@@ -77,7 +84,7 @@
 
 /*--------------------------------------------------------------------*/
 
-TAILQ_HEAD(smfhead, smf);
+VTAILQ_HEAD(smfhead, smf);
 
 struct smf {
 	unsigned		magic;
@@ -91,8 +98,8 @@ struct smf {
 	off_t			offset;
 	unsigned char		*ptr;
 
-	TAILQ_ENTRY(smf)	order;
-	TAILQ_ENTRY(smf)	status;
+	VTAILQ_ENTRY(smf)	order;
+	VTAILQ_ENTRY(smf)	status;
 	struct smfhead		*flist;
 };
 
@@ -112,27 +119,43 @@ struct smf_sc {
 static void
 smf_calcsize(struct smf_sc *sc, const char *size, int newfile)
 {
-	uintmax_t l;
+#if defined(HAVE_SYS_STATVFS_H)
+	struct statvfs fsst;
+#elif defined(HAVE_SYS_MOUNT_H) || defined(HAVE_SYS_VFS_H)
+	struct statfs fsst;
+#else
+#error no struct statfs / struct statvfs
+#endif
+	uintmax_t l, fssize;
 	unsigned bs;
 	char suff[2];
-	int i, expl;
+	int i, explicit;
 	off_t o;
-	struct statfs fsst;
 	struct stat st;
 
+	AN(sc);
 	AZ(fstat(sc->fd, &st));
+	xxxassert(S_ISREG(st.st_mode));
+
+#if defined(HAVE_SYS_STATVFS_H)
+	AZ(fstatvfs(sc->fd, &fsst));
+#elif defined(HAVE_SYS_MOUNT_H) || defined(HAVE_SYS_VFS_H)
 	AZ(fstatfs(sc->fd, &fsst));
+#else
+#error no struct statfs / struct statvfs
+#endif
 
 	/* We use units of the larger of filesystem blocksize and pagesize */
 	bs = sc->pagesize;
 	if (bs < fsst.f_bsize)
 		bs = fsst.f_bsize;
-
-	xxxassert(S_ISREG(st.st_mode));
+	xxxassert(bs % sc->pagesize == 0);
+	xxxassert(bs % fsst.f_bsize == 0);
+	fssize = fsst.f_bsize * fsst.f_bavail;
 
 	i = sscanf(size, "%ju%1s", &l, suff); /* can return -1, 0, 1 or 2 */
 
-	expl = i;
+	explicit = i;
 	if (i == 0) {
 		fprintf(stderr,
 		    "Error: (-sfile) size \"%s\" not understood\n", size);
@@ -169,7 +192,7 @@ smf_calcsize(struct smf_sc *sc, const char *size, int newfile)
 				l *= (uintmax_t)(1024UL * 1024UL) *
 				    (uintmax_t)(1024UL * 1024UL);
 			else if (suff[0] == '%') {
-				l *= fsst.f_bsize * fsst.f_bavail;
+				l *= fssize;
 				l /= 100;
 			}
 		}
@@ -190,14 +213,14 @@ smf_calcsize(struct smf_sc *sc, const char *size, int newfile)
 
 		if (l < st.st_size) {
 			AZ(ftruncate(sc->fd, l));
-		} else if (l - st.st_size > fsst.f_bsize * fsst.f_bavail) {
-			l = ((uintmax_t)fsst.f_bsize * fsst.f_bavail * 80) / 100;
+		} else if (l - st.st_size > fssize) {
+			l = fssize * 80 / 100;
 			fprintf(stderr, "WARNING: storage file size reduced"
 			    " to %ju (80%% of available disk space)\n", l);
 		}
 	}
 
-	/* round down to of filesystem blocksize or pagesize */
+	/* round down to multiple of filesystem blocksize or pagesize */
 	l -= (l % bs);
 
 	if (l < MINPAGES * (uintmax_t)sc->pagesize) {
@@ -207,7 +230,7 @@ smf_calcsize(struct smf_sc *sc, const char *size, int newfile)
 		exit (2);
 	}
 
-	if (expl < 3 && sizeof(void *) == 4 && l > INT32_MAX) {
+	if (explicit < 3 && sizeof(void *) == 4 && l > INT32_MAX) {
 		fprintf(stderr,
 		    "NB: Limiting size to 2GB on 32 bit architecture to"
 		    " prevent running out of\naddress space."
@@ -244,19 +267,18 @@ smf_init(struct stevedore *parent, const char *spec)
 
 	sc = calloc(sizeof *sc, 1);
 	XXXAN(sc);
-	TAILQ_INIT(&sc->order);
+	VTAILQ_INIT(&sc->order);
 	for (u = 0; u < NBUCKET; u++)
-		TAILQ_INIT(&sc->free[u]);
-	TAILQ_INIT(&sc->used);
+		VTAILQ_INIT(&sc->free[u]);
+	VTAILQ_INIT(&sc->used);
 	sc->pagesize = getpagesize();
 
 	parent->priv = sc;
 
 	/* If no size specified, use 50% of filesystem free space */
 	if (spec == NULL || *spec == '\0')
-		spec = "/tmp,50%";
-
-	if (strchr(spec, ',') == NULL)
+		asprintf(&p, ".,50%%");
+	else if (strchr(spec, ',') == NULL)
 		asprintf(&p, "%s,", spec);
 	else
 		p = strdup(spec);
@@ -310,6 +332,7 @@ smf_init(struct stevedore *parent, const char *spec)
 		return;
 	}
 
+
 	asprintf(&q, "%s/varnish.XXXXXX", p);
 	XXXAN(q);
 	sc->fd = mkstemp(q);
@@ -324,6 +347,7 @@ smf_init(struct stevedore *parent, const char *spec)
 	XXXAN(sc->filename);
 	free(q);
 	smf_initfile(sc, size, 1);
+	free(p);
 }
 
 /*--------------------------------------------------------------------
@@ -348,7 +372,7 @@ insfree(struct smf_sc *sc, struct smf *sp)
 	}
 	sp->flist = &sc->free[b];
 	ns = b * sc->pagesize;
-	TAILQ_FOREACH(sp2, sp->flist, status) {
+	VTAILQ_FOREACH(sp2, sp->flist, status) {
 		assert(sp2->size >= ns);
 		assert(sp2->alloc == 0);
 		assert(sp2->flist == sp->flist);
@@ -356,9 +380,9 @@ insfree(struct smf_sc *sc, struct smf *sp)
 			break;
 	}
 	if (sp2 == NULL)
-		TAILQ_INSERT_TAIL(sp->flist, sp, status);
+		VTAILQ_INSERT_TAIL(sp->flist, sp, status);
 	else
-		TAILQ_INSERT_BEFORE(sp2, sp, status);
+		VTAILQ_INSERT_BEFORE(sp2, sp, status);
 }
 
 static void
@@ -376,7 +400,7 @@ remfree(struct smf_sc *sc, struct smf *sp)
 		VSL_stats->n_smf_frag--;
 	}
 	assert(sp->flist == &sc->free[b]);
-	TAILQ_REMOVE(sp->flist, sp, status);
+	VTAILQ_REMOVE(sp->flist, sp, status);
 	sp->flist = NULL;
 }
 
@@ -395,12 +419,12 @@ alloc_smf(struct smf_sc *sc, size_t bytes)
 	if (b >= NBUCKET)
 		b = NBUCKET - 1;
 	for (sp = NULL; b < NBUCKET - 1; b++) {
-		sp = TAILQ_FIRST(&sc->free[b]);
+		sp = VTAILQ_FIRST(&sc->free[b]);
 		if (sp != NULL)
 			break;
 	}
 	if (sp == NULL) {
-		TAILQ_FOREACH(sp, &sc->free[NBUCKET -1], status)
+		VTAILQ_FOREACH(sp, &sc->free[NBUCKET -1], status)
 			if (sp->size >= bytes)
 				break;
 	}
@@ -412,7 +436,7 @@ alloc_smf(struct smf_sc *sc, size_t bytes)
 
 	if (sp->size == bytes) {
 		sp->alloc = 1;
-		TAILQ_INSERT_TAIL(&sc->used, sp, status);
+		VTAILQ_INSERT_TAIL(&sc->used, sp, status);
 		return (sp);
 	}
 
@@ -428,14 +452,14 @@ alloc_smf(struct smf_sc *sc, size_t bytes)
 
 	sp2->size = bytes;
 	sp2->alloc = 1;
-	TAILQ_INSERT_BEFORE(sp, sp2, order);
-	TAILQ_INSERT_TAIL(&sc->used, sp2, status);
+	VTAILQ_INSERT_BEFORE(sp, sp2, order);
+	VTAILQ_INSERT_TAIL(&sc->used, sp2, status);
 	insfree(sc, sp);
 	return (sp2);
 }
 
 /*--------------------------------------------------------------------
- * Free a range.  Attemt merge forward and backward, then sort into
+ * Free a range.  Attempt merge forward and backward, then sort into
  * free list according to age.
  */
 
@@ -449,29 +473,29 @@ free_smf(struct smf *sp)
 	assert(sp->alloc != 0);
 	assert(sp->size > 0);
 	assert(!(sp->size % sc->pagesize));
-	TAILQ_REMOVE(&sc->used, sp, status);
+	VTAILQ_REMOVE(&sc->used, sp, status);
 	sp->alloc = 0;
 
-	sp2 = TAILQ_NEXT(sp, order);
+	sp2 = VTAILQ_NEXT(sp, order);
 	if (sp2 != NULL &&
 	    sp2->alloc == 0 &&
 	    (sp2->ptr == sp->ptr + sp->size) &&
 	    (sp2->offset == sp->offset + sp->size)) {
 		sp->size += sp2->size;
-		TAILQ_REMOVE(&sc->order, sp2, order);
+		VTAILQ_REMOVE(&sc->order, sp2, order);
 		remfree(sc, sp2);
 		free(sp2);
 		VSL_stats->n_smf--;
 	}
 
-	sp2 = TAILQ_PREV(sp, smfhead, order);
+	sp2 = VTAILQ_PREV(sp, smfhead, order);
 	if (sp2 != NULL &&
 	    sp2->alloc == 0 &&
 	    (sp->ptr == sp2->ptr + sp2->size) &&
 	    (sp->offset == sp2->offset + sp2->size)) {
 		remfree(sc, sp2);
 		sp2->size += sp->size;
-		TAILQ_REMOVE(&sc->order, sp, order);
+		VTAILQ_REMOVE(&sc->order, sp, order);
 		free(sp);
 		VSL_stats->n_smf--;
 		sp = sp2;
@@ -505,8 +529,8 @@ trim_smf(struct smf *sp, size_t bytes)
 	sp->size = bytes;
 	sp2->ptr += bytes;
 	sp2->offset += bytes;
-	TAILQ_INSERT_AFTER(&sc->order, sp, sp2, order);
-	TAILQ_INSERT_TAIL(&sc->used, sp2, status);
+	VTAILQ_INSERT_AFTER(&sc->order, sp, sp2, order);
+	VTAILQ_INSERT_TAIL(&sc->used, sp2, status);
 	free_smf(sp2);
 }
 
@@ -532,16 +556,16 @@ new_smf(struct smf_sc *sc, unsigned char *ptr, off_t off, size_t len)
 	sp->offset = off;
 	sp->alloc = 1;
 
-	TAILQ_FOREACH(sp2, &sc->order, order) {
+	VTAILQ_FOREACH(sp2, &sc->order, order) {
 		if (sp->ptr < sp2->ptr) {
-			TAILQ_INSERT_BEFORE(sp2, sp, order);
+			VTAILQ_INSERT_BEFORE(sp2, sp, order);
 			break;
 		}
 	}
 	if (sp2 == NULL)
-		TAILQ_INSERT_TAIL(&sc->order, sp, order);
+		VTAILQ_INSERT_TAIL(&sc->order, sp, order);
 
-	TAILQ_INSERT_HEAD(&sc->used, sp, status);
+	VTAILQ_INSERT_HEAD(&sc->used, sp, status);
 
 	free_smf(sp);
 }
@@ -606,6 +630,8 @@ smf_open(struct stevedore *st)
 	if (sum < MINPAGES * (uintmax_t)getpagesize())
 		exit (2);
 	MTX_INIT(&sc->mtx);
+
+	VSL_stats->sm_bfree += sc->filesize;
 }
 
 /*--------------------------------------------------------------------*/
@@ -620,8 +646,16 @@ smf_alloc(struct stevedore *st, size_t size)
 	size += (sc->pagesize - 1);
 	size &= ~(sc->pagesize - 1);
 	LOCK(&sc->mtx);
+	VSL_stats->sm_nreq++;
 	smf = alloc_smf(sc, size);
+	if (smf == NULL) {
+		UNLOCK(&sc->mtx);
+		return (NULL);
+	}
 	CHECK_OBJ_NOTNULL(smf, SMF_MAGIC);
+	VSL_stats->sm_nobj++;
+	VSL_stats->sm_balloc += smf->size;
+	VSL_stats->sm_bfree -= smf->size;
 	UNLOCK(&sc->mtx);
 	XXXAN(smf);
 	assert(smf->size == size);
@@ -655,6 +689,8 @@ smf_trim(struct storage *s, size_t size)
 	size &= ~(sc->pagesize - 1);
 	if (smf->size > size) {
 		LOCK(&sc->mtx);
+		VSL_stats->sm_balloc -= (smf->size - size);
+		VSL_stats->sm_bfree += (smf->size - size);
 		trim_smf(smf, size);
 		assert(smf->size == size);
 		UNLOCK(&sc->mtx);
@@ -674,6 +710,9 @@ smf_free(struct storage *s)
 	CAST_OBJ_NOTNULL(smf, s->priv, SMF_MAGIC);
 	sc = smf->sc;
 	LOCK(&sc->mtx);
+	VSL_stats->sm_nobj--;
+	VSL_stats->sm_balloc -= smf->size;
+	VSL_stats->sm_bfree += smf->size;
 	free_smf(smf);
 	UNLOCK(&sc->mtx);
 }
@@ -708,17 +747,17 @@ dumpit(void)
 	return (0);
 	printf("----------------\n");
 	printf("Order:\n");
-	TAILQ_FOREACH(s, &sc->order, order) {
+	VTAILQ_FOREACH(s, &sc->order, order) {
 		printf("%10p %12ju %12ju %12ju\n",
 		    s, s->offset, s->size, s->offset + s->size);
 	}
 	printf("Used:\n");
-	TAILQ_FOREACH(s, &sc->used, status) {
+	VTAILQ_FOREACH(s, &sc->used, status) {
 		printf("%10p %12ju %12ju %12ju\n",
 		    s, s->offset, s->size, s->offset + s->size);
 	}
 	printf("Free:\n");
-	TAILQ_FOREACH(s, &sc->free, status) {
+	VTAILQ_FOREACH(s, &sc->free, status) {
 		printf("%10p %12ju %12ju %12ju\n",
 		    s, s->offset, s->size, s->offset + s->size);
 	}
