@@ -1,5 +1,5 @@
 /*-
- * Copyright (c) 2000 Poul-Henning Kamp and Dag-Erling Codan Smrgrav
+ * Copyright (c) 2000 Poul-Henning Kamp and Dag-Erling Smørgrav
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -25,7 +25,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * $Id: vsb.h 1096 2006-09-20 07:50:27Z des $
+ * $Id: vsb.h 1818 2007-08-09 12:12:12Z des $
  * $FreeBSD: src/sys/sys/vsb.h,v 1.14 2004/07/09 11:35:30 des Exp $
  */
 
@@ -50,7 +50,9 @@ struct vsb {
 	int		 s_flags;	/* flags */
 };
 
-__BEGIN_DECLS
+#ifdef __cplusplus
+extern "C" {
+#endif
 /*
  * API functions
  */
@@ -73,6 +75,8 @@ char		*vsb_data(struct vsb *);
 int		 vsb_len(struct vsb *);
 int		 vsb_done(struct vsb *);
 void		 vsb_delete(struct vsb *);
-__END_DECLS
+#ifdef __cplusplus
+};
+#endif
 
 #endif
