@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006 Linpro AS
+ * Copyright (c) 2006-2007 Linpro AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -14,7 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  *
- * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
+ * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
@@ -26,19 +26,21 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: shmlog.c 1096 2006-09-20 07:50:27Z des $
+ * $Id: shmlog.c 1588 2007-06-28 10:25:47Z des $
  */
 
-#include <stdio.h>
-#include <errno.h>
-#include <ctype.h>
-#include <string.h>
-#include <stdlib.h>
-#include <unistd.h>
-#include <fcntl.h>
-#include <regex.h>
 #include <sys/mman.h>
+
 #include <assert.h>
+#include <ctype.h>
+#include <errno.h>
+#include <fcntl.h>
+#include <limits.h>
+#include <regex.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <unistd.h>
 
 #include "shmlog.h"
 #include "miniobj.h"
@@ -88,6 +90,7 @@ struct VSL_data {
 
 static int vsl_fd;
 static struct shmloghead *vsl_lh;
+static char vsl_name[PATH_MAX];
 
 static int vsl_nextlog(struct VSL_data *vd, unsigned char **pp);
 
@@ -102,29 +105,40 @@ const char *VSL_tags[256] = {
 /*--------------------------------------------------------------------*/
 
 static int
-vsl_shmem_map(void)
+vsl_shmem_map(const char *varnish_name)
 {
 	int i;
 	struct shmloghead slh;
+	char dirname[PATH_MAX], logname[PATH_MAX];
 
 	if (vsl_lh != NULL)
 		return (0);
 
-	vsl_fd = open(SHMLOG_FILENAME, O_RDONLY);
+	if (varnish_instance(varnish_name, vsl_name,
+	    sizeof vsl_name, dirname, sizeof dirname) != 0) {
+		fprintf(stderr, "Invalid instance name: %s\n",
+		    strerror(errno));
+		return (1);
+	}
+
+	/* XXX check overflow */
+	snprintf(logname, sizeof logname, "%s/%s", dirname, SHMLOG_FILENAME);
+
+	vsl_fd = open(logname, O_RDONLY);
 	if (vsl_fd < 0) {
 		fprintf(stderr, "Cannot open %s: %s\n",
-		    SHMLOG_FILENAME, strerror(errno));
+		    logname, strerror(errno));
 		return (1);
 	}
 	i = read(vsl_fd, &slh, sizeof slh);
 	if (i != sizeof slh) {
 		fprintf(stderr, "Cannot read %s: %s\n",
-		    SHMLOG_FILENAME, strerror(errno));
+		    logname, strerror(errno));
 		return (1);
 	}
 	if (slh.magic != SHMLOGHEAD_MAGIC) {
 		fprintf(stderr, "Wrong magic number in file %s\n",
-		    SHMLOG_FILENAME);
+		    logname);
 		return (1);
 	}
 
@@ -132,7 +146,7 @@ vsl_shmem_map(void)
 	    PROT_READ, MAP_SHARED|MAP_HASSEMAPHORE, vsl_fd, 0);
 	if (vsl_lh == MAP_FAILED) {
 		fprintf(stderr, "Cannot mmap %s: %s\n",
-		    SHMLOG_FILENAME, strerror(errno));
+		    logname, strerror(errno));
 		return (1);
 	}
 	return (0);
@@ -167,7 +181,7 @@ VSL_Select(struct VSL_data *vd, unsigned tag)
 /*--------------------------------------------------------------------*/
 
 int
-VSL_OpenLog(struct VSL_data *vd)
+VSL_OpenLog(struct VSL_data *vd, const char *varnish_name)
 {
 	unsigned char *p;
 
@@ -175,8 +189,8 @@ VSL_OpenLog(struct VSL_data *vd)
 	if (vd->fi != NULL)
 		return (0);
 
-	if (vsl_shmem_map())
-		return (1);
+	if (vsl_shmem_map(varnish_name))
+		return (-1);
 
 	vd->head = vsl_lh;
 	vd->logstart = (unsigned char *)vsl_lh + vsl_lh->start;
@@ -265,6 +279,7 @@ VSL_NextLog(struct VSL_data *vd, unsigned char **pp)
 			vd->map[u] &= ~M_BACKEND;
 			break;
 		case SLT_BackendOpen:
+		case SLT_BackendXID:
 			vd->map[u] |= M_BACKEND;
 			vd->map[u] &= ~M_CLIENT;
 			break;
@@ -284,14 +299,14 @@ VSL_NextLog(struct VSL_data *vd, unsigned char **pp)
 		if (vd->regincl != NULL) {
 			rm.rm_so = 0;
 			rm.rm_eo = p[1];
-			i = regexec(vd->regincl, p + 4, 1, &rm, 0);
+			i = regexec(vd->regincl, (char *)p + 4, 1, &rm, 0);
 			if (i == REG_NOMATCH)
 				continue;
 		}
 		if (vd->regexcl != NULL) {
 			rm.rm_so = 0;
 			rm.rm_eo = p[1];
-			i = regexec(vd->regexcl, p + 4, 1, &rm, 0);
+			i = regexec(vd->regexcl, (char *)p + 4, 1, &rm, 0);
 			if (i != REG_NOMATCH)
 				continue;
 		}
@@ -318,7 +333,7 @@ VSL_Dispatch(struct VSL_data *vd, vsl_handler *func, void *priv)
 		if (func(priv,
 		    p[0], u, p[1],
 		    vd->map[u] & (VSL_S_CLIENT|VSL_S_BACKEND),
-		    p + 4))
+		    (char *)p + 4))
 			return (1);
 	}
 }
@@ -326,7 +341,7 @@ VSL_Dispatch(struct VSL_data *vd, vsl_handler *func, void *priv)
 /*--------------------------------------------------------------------*/
 
 int
-VSL_H_Print(void *priv, unsigned tag, unsigned fd, unsigned len, unsigned spec, const char *ptr)
+VSL_H_Print(void *priv, enum shmlogtag tag, unsigned fd, unsigned len, unsigned spec, const char *ptr)
 {
 	FILE *fo = priv;
 
@@ -473,11 +488,17 @@ VSL_Arg(struct VSL_data *vd, int arg, const char *opt)
 }
 
 struct varnish_stats *
-VSL_OpenStats(void)
+VSL_OpenStats(const char *varnish_name)
 {
 
-	if (vsl_shmem_map())
+	if (vsl_shmem_map(varnish_name))
 		return (NULL);
 	return (&vsl_lh->stats);
 }
 
+const char *
+VSL_Name(void)
+{
+
+	return (vsl_name);
+}
